unit Config;

interface

uses
 Messages, Windows;

const
 WM_FULL     = WM_USER + 5200;

 cmNoCommand = 1100;
 cmSmall     = 1101;
 cmFull      = 1102;
 cmHosts     = 1103;
 cmUsers     = 1104;
 cmExitImm   = 1105;

 AppBase     = 'HKEY_LOCAL_MACHINE\Software\Drt\NTPOP\';

 OneSecond   = (1/24/60/60);

type
 TCfg = record
   Port   : word;
   Spool  : string;
   Run    : boolean;
   LogLvl : integer;
   TOut   : integer;
   Show   : boolean;
   end;

var
 Cfg : TCfg;

procedure LoadConfiguration;
procedure SaveConfiguration;

procedure bzero(var b; n : word);
function FLM(Level : integer; Msg : string) : string;
function getword(s : string; n : byte) : string;
function getend(s : string; n : byte) : string;
function AllTrim(s : string) : string;
function IPIsValid(AIP : string) : boolean;
function GetIPPart(AIP : string; n : integer) : string;
function Empty(S : string) : boolean;
function ReplaceExtension(fn,ext : string) : string;
function Unhex(s : string) : integer;
function AddSlash(s : string) : string;

implementation

uses
 Reg, Lsts, SysUtils;

procedure Defaults;
begin
Cfg.Port:=110;
Cfg.Spool:='c:\inetpub\mailroot\drop';
Cfg.Run:=false;
Cfg.LogLvl:=2;
Cfg.TOut:=60;
Cfg.Show:=true;
end;

procedure SaveConfiguration;
var
 i,a,d : integer;
begin
RegistryDeleteKey(AppBase);
RegistrySetValue(AppBase+'Port',Cfg.Port);
RegistrySetValue(AppBase+'LogLevel',Cfg.LogLvl);
RegistrySetValue(AppBase+'TimeOut',Cfg.TOut);
RegistrySetValue(AppBase+'SpoolDir',Cfg.Spool);
RegistrySetValue(AppBase+'RunOnStart',Cfg.Run,sizeof(boolean));
RegistrySetValue(AppBase+'ShowOnStart',Cfg.Show,sizeof(boolean));
if assigned(HostList) then
  begin
  a:=0; d:=0;
  for i:=0 to HostList.Count-1 do
    if HostList.AD[i] then
      begin
      RegistrySetValue(AppBase+'Hosts\Allow'+IntToStr(a),HostList.Mask[i]);
      inc(a);
      end
    else
      begin
      RegistrySetValue(AppBase+'Hosts\Deny'+IntToStr(d),HostList.Mask[i]);
      inc(d);
      end;
  end;
if assigned(DomainList) then
  begin
  for i:=0 to DomainList.Count-1 do
    RegistrySetValue(AppBase+'Domains\Domain'+IntToStr(i),DomainList[i]);
  end;
if assigned(UserList) then
  begin
  for i:=0 to UserList.Count-1 do
    RegistrySetValue(AppBase+'Users\User'+IntToStr(i),UserList.Data[i]);
  end;
end;

procedure LoadConfiguration;
var
 i : dword;
 s : string;
 b : boolean;
 j : integer;
begin
Defaults;
if RegistryGetValue(AppBase+'Port',i) then Cfg.Port:=i;
if RegistryGetValue(AppBase+'LogLevel',i) then Cfg.LogLvl:=i;
if RegistryGetValue(AppBase+'TimeOut',i) then Cfg.TOut:=i;
if RegistryGetValue(AppBase+'SpoolDir',s) then Cfg.Spool:=s;
j:=sizeof(boolean);
if RegistryGetValue(AppBase+'RunOnStart',b,j) then Cfg.Run:=b;
j:=sizeof(boolean);
if RegistryGetValue(AppBase+'ShowOnStart',b,j) then Cfg.Show:=b;
if Assigned(HostList) then
  begin
  HostList.Clear;
  j:=0;
  while RegistryGetValue(AppBase+'Hosts\Deny'+IntToStr(j),s) do
    begin
    if IPIsValid(s) then HostList.Add(s,false);
    inc(j);
    end;
  j:=0;
  while RegistryGetValue(AppBase+'Hosts\Allow'+IntToStr(j),s) do
    begin
    if IPIsValid(s) then HostList.Add(s,true);
    inc(j);
    end;
  end;
if Assigned(DomainList) then
  begin
  j:=0;
  while RegistryGetValue(AppBase+'Domains\Domain'+IntToStr(j),s) do
    begin
    DomainList.Add(s);
    inc(j);
    end;
  end;
if Assigned(UserList) then
  begin
  j:=0;
  while RegistryGetValue(AppBase+'Users\User'+IntToStr(j),s) do
    begin
    UserList.Add(s);
    inc(j);
    end;
  end;
end;

type
 tb = array [0..$fff0] of byte;

procedure bzero(var b; n : word);
begin
while n > 0 do
  begin
  tb(b)[n-1]:=0;
  dec(n);
  end;
end;

function FLM(Level : integer; Msg : string) : string;
begin
result:='';
if Level > Cfg.LogLvl then exit;
result:=DateTimeToStr(Now)+' %s '+Msg;
end;

function getword(s : string; n : byte) : string;
var
 i : byte;
begin
getword:='';
s:=alltrim(s);
for i:=2 to n do
  if pos(' ',s) > 0 then
    s:=alltrim(copy(s,pos(' ',s),255))
  else
    s:='';
if s <> '' then
  begin
  if pos(' ',s) > 0 then
    getword:=alltrim(copy(s,1,pos(' ',s)))
  else
    getword:=s;
  end;
end;

function getend(s : string; n : byte) : string;
var
 i : byte;
begin
getend:='';
s:=alltrim(s);
for i:=2 to n do
  if pos(' ',s) > 0 then
    s:=alltrim(copy(s,pos(' ',s),255))
  else
    s:='';
getend:=s;
end;

function AllTrim(s : string) : string;
var
 i : integer;
begin
result:='';
if length(s) = 0 then exit;
i:=1;
while i <= length(s) do
  begin
  if s[i] <> ' ' then break;
  inc(i);
  end;
s:=copy(s,i,1000);
if length(s) = 0 then exit;
i:=length(s);
while (i > 0) do
  begin
  if s[i] <> ' ' then break;
  dec(i);
  end;
setlength(s,i);
result:=s;
end;

function IPIsValid(AIP : string) : boolean;
var
 i,j : integer;
 s : string;
begin
result:=false;
for i:=1 to 4 do
  begin
  s:=GetIPPart(AIP,i);
  j:=StrToIntDef(s,-1);
  if (s <> '*') and (empty(s) or (j < 0) or (j > 255)) then exit;
  end;
result:=true;
end;

function GetIPPart(AIP : string; n : integer) : string;
var
 i : integer;
begin
result:='';
while n > 1 do
  begin
  i:=pos('.',AIP);
  if empty(AIP) or (i = 0) then exit;
  AIP:=copy(AIP,i+1,20);
  dec(n);
  end;
if empty(AIP) then exit;
i:=pos('.',AIP);
if i > 0 then
  AIP:=copy(AIP,1,i-1);
result:=alltrim(AIP);
end;

function Empty(S : string) : boolean;
var
 i : integer;
begin
result:=false;
for i:=1 to length(S) do
  if not (S[i] in [' ',#9]) then exit;
result:=true;
end;

function AddSlash(s : string) : string;
begin
AddSlash:='';
if empty(s) then exit;
if length(s) <> 2 then
  begin
  if not (s[length(s)] in ['\','/']) then
    s:=s+'\';
  end
else
  if s[2] <> ':' then
    s:=s+'\';
AddSlash:=s;
end;

function ReplaceExtension(fn,ext : string) : string;
var
 i : integer;
begin
result:='';
if empty(fn) then exit;
i:=length(fn);
while (i > 0) and not (fn[i] in ['.','/','\',':']) do dec(i);
if fn[i] = '.' then
  begin
  if empty(ext) then
    result:=copy(fn,1,i-1)
  else
    result:=copy(fn,1,i)+ext;
  end
else
  begin
  if empty(ext) then
    result:=fn
  else
    result:=fn+'.'+ext;
  end;
end;

function Unhex(s : string) : integer;
var
 i : integer;
begin
result:=0;
for i:=1 to length(s) do
  begin
  if s[i] in ['0'..'9'] then
    result:=result*16+StrToIntDef(s[i],0)
  else
    result:=result*16+(byte(s[i])-$37);
  end;
end;

end.
