unit Folder;

interface

uses
 Classes, Windows;

type
 TMsgData = class
   constructor Create(AMSGID : string; ASize : longint; ARepNo : integer);
 public
   MSGID   : string;
   Size    : longint;
   RepNo   : integer;
   Deleted : boolean;
   end;

 TFolder = class(TStringList)
   constructor Create(AUsername : string);
   destructor Destroy; override;
 private
   Mtx      : THANDLE;
   Username : string;
   T        : textfile;
   MsgState : boolean;
   MsgTop   : boolean;
   function UserIsMe(AAddress : string) : boolean;
   function GetSize : integer;
   function GetMsgSize(i : integer) : integer;
   function GetMsgDeleted(i : integer) : boolean;
   function GetMsgMSGID(i : integer) : string;
   procedure CutList(AIndex : integer);
 public
   Error    : integer;
   property Size : integer read GetSize;
   property MsgSize[i : integer] : integer read GetMsgSize;
   property MsgDeleted[i : integer] : boolean read GetMsgDeleted;
   property MsgMSGID[i : integer] : string read GetMsgMSGID;
   function MsgOpen(ANumber : integer) : boolean;
   function MsgGetLine(var AOut : string) : boolean;
   function MsgClose : boolean;
   function MsgDelete(ANumber : integer) : boolean;
   procedure Reset;
   procedure Update;
   end;

implementation

uses
 Config, SysUtils, UpdateMD5, Logging, Lsts;

{ ================================== TFolder ================================= }

function TFolder.UserIsMe(AAddress : string) : boolean;
var
 un,dm : string;
 i : integer;
begin
result:=false;
i:=pos('@',AAddress);
if i = 0 then exit;
un:=copy(AAddress,1,i-1);
dm:=copy(AAddress,i+1,1000);
if DomainList.Match(dm) and (uppercase(un) = uppercase(Username)) then result:=true;
end;

constructor TFolder.Create(AUsername : string);
var
 s,s1 : string;
 SR : TSearchRec;
 i : integer;
 TMSGID : string;
 TRepNo : integer;
 TMyMsg : boolean;
 SMsg : boolean;
 T : textfile;
begin
inherited Create;
Error:=0;
Username:=AUsername;
MsgState:=false;
s:='Folder_Mutex_For_'+UserName;
Mtx:=CreateMutex(nil,true,pchar(s));
if Mtx = 0 then
  begin
  Error:=1;
  exit;
  end;
s:=AddSlash(Cfg.Spool)+'*.*';
i:=FindFirst(pchar(s),faArchive,SR);
while i = 0 do
  begin
  assignfile(T,AddSlash(Cfg.Spool)+SR.Name);
  try
    try
      system.reset(T);
      s1:='';
      TRepNo:=0;
      TMyMsg:=false;
      SMsg:=false;
      while not eof(T) do
        begin
        readln(T,s);
        if empty(s) then break;
        if SMsg then
          s1:=s1+s
        else
          begin
          if (copy(s,1,12) <> 'x-receiver: ') and (copy(s,1,10) <> 'x-sender: ') then
            begin
            SMsg:=true;
            s1:=s;
            end
          else
            begin
            if copy(s,1,12) = 'x-receiver: ' then
              begin
              if UserIsMe(copy(s,13,1000)) then
                TMyMsg:=true
              else
                inc(TRepNo);
              end;
            end;
          end;
        end;
      TMSGID:=GetMD5Str(s1[1],length(s1));
      if TMyMsg then
        AddObject(AddSlash(Cfg.Spool)+SR.Name,TMsgData.Create(TMSGID,SR.Size,TRepNo));
    except
      on E: Exception do
        Log(FLM(2,E.Message));
      end;
  finally
    try
      closefile(T);
    except
      end;
    end;
  i:=FindNext(SR);
  end;
FindClose(SR);
end;

destructor TFolder.Destroy;
begin
ReleaseMutex(Mtx);
CloseHandle(Mtx);
inherited Destroy;
end;

function TFolder.GetSize : integer;
var
 i : integer;
begin
result:=0;
for i:=0 to count-1 do
  result:=result+TMsgData(objects[i]).Size;
end;

function TFolder.GetMsgSize(i : integer) : integer;
begin
result:=-1;
if (i < 0) or (i >= count) then exit;
result:=TMsgData(objects[i]).Size;
end;

function TFolder.GetMsgDeleted(i : integer) : boolean;
begin
result:=true;
if (i < 0) or (i >= count) then exit;
result:=TMsgData(objects[i]).Deleted;
end;

function TFolder.GetMsgMSGID(i : integer) : string;
begin
result:='';
if (i < 0) or (i >= count) then exit;
result:=TMsgData(objects[i]).MSGID;
end;

function TFolder.MsgOpen(ANumber : integer) : boolean;
begin
if MsgState then MsgClose;
MsgState:=false;
result:=false;
if (ANumber < 0) or (ANumber >= count) then exit;
assignfile(T,strings[ANumber]);
try
  system.reset(T);
  MsgState:=true;
  MsgTop:=false;
except
  end;
result:=MsgState;
end;

function TFolder.MsgGetLine(var AOut : string) : boolean;
begin
result:=false;
if not MsgState then exit;
try
  if eof(T) then exit;
  if MsgTop then
    begin
    readln(T,AOut);
    result:=true;
    end
  else
    begin
    while not MsgTop and not eof(T) do
      begin
      readln(T,AOut);
      if (copy(AOut,1,12) <> 'x-receiver: ') and (copy(AOut,1,10) <> 'x-sender: ') then
        MsgTop:=true;
      end;
    result:=MsgTop;
    end;
except
  end;
end;

function TFolder.MsgClose : boolean;
begin
result:=false;
try
  if MsgState then closefile(T);
  result:=true;
except
  end;
MsgState:=false;
end;

function TFolder.MsgDelete(ANumber : integer) : boolean;
begin
result:=false;
if (ANumber < 0) or (ANumber >= count) then exit;
if not MsgDeleted[Anumber] then
  begin
  TMsgData(objects[ANumber]).Deleted:=true;
  result:=true;
  end;
end;

procedure TFolder.Reset;
var
 i : integer;
begin
for i:=0 to count-1 do
  TMsgData(objects[i]).Deleted:=false;
end;

procedure TFolder.Update;
var
 i : integer;
begin
for i:=0 to count-1 do
  begin
  try
    if TMsgData(objects[i]).Deleted then
      if TMsgData(objects[i]).RepNo = 0 then
        deletefile(strings[i])
      else
        CutList(i);
  except
    end;
  end;
end;

procedure TFolder.CutList(AIndex : integer);
var
 T1,T2 : textfile;
 s : string;
 Mtx : THANDLE;
 i : integer;
 Top : boolean;
 T1Opn : boolean;
 T2Opn : boolean;
 Rcp : integer;
begin
Mtx:=0;
T1Opn:=false;
T2Opn:=false;
try
  s:='FileLockFor_'+TMsgData(objects[AIndex]).MSGID;
  i:=10;
  repeat
    Mtx:=CreateMutex(nil,true,pchar(s));
    if Mtx = 0 then sleep(1000);
    dec(i);
    until (Mtx > 0) or (i = 0);
  if Mtx = 0 then Log(FLM(2,'Can''t lock file for updating'));
  assignfile(T1,strings[AIndex]);
  assignfile(T2,AddSlash(ExtractFilePath(strings[AIndex]))+TMsgData(objects[AIndex]).MSGID);
  try
    system.reset(T1); T1Opn:=true;
    system.rewrite(T2); T2Opn:=true;
    Top:=false; Rcp:=0;
    while not eof(T1) do
      begin
      readln(T1,s);
      if Top then
        writeln(T2,s)
      else
        begin
        if (copy(s,1,12) <> 'x-receiver: ') and (copy(s,1,10) <> 'x-sender: ') then
          begin
          if Rcp = 0 then break;
          Top:=true;
          writeln(T2,s);
          end
        else
          begin
          if (copy(s,1,12) = 'x-receiver: ') then
            begin
            if not UserIsMe(copy(s,13,1000)) then
              begin
              writeln(T2,s);
              inc(Rcp);
              end;
            end
          else
            writeln(T2,s);
          end;
        end;
      end;
    closefile(T1); T1Opn:=false;
    closefile(T2); T2Opn:=false;
    if Rcp = 0 then
      begin
      deletefile(strings[AIndex]);
      deletefile(AddSlash(ExtractFilePath(strings[AIndex]))+TMsgData(objects[AIndex]).MSGID);
      end
    else
      begin
      if deletefile(strings[AIndex]) then
        rename(T2,strings[AIndex])
      else
        begin
        deletefile(AddSlash(ExtractFilePath(strings[AIndex]))+TMsgData(objects[AIndex]).MSGID);
        Log(FLM(1,'Can''t rename file "'+strings[AIndex]+'" to "'+AddSlash(ExtractFilePath(strings[AIndex]))+TMsgData(objects[AIndex]).MSGID+'"'));
        end;
      end;
  except
    on E: Exception do
      Log(FLM(1,E.Message));
    end;
finally
  if T1Opn then
    try
      closefile(T1);
    except
      end;
  if T2Opn then
    try
      closefile(T2);
    except
      end;
  if Mtx > 0 then
    begin
    ReleaseMutex(Mtx);
    CloseHandle(Mtx);
    end;
  end;
end;

{ ================================== TMsgData ================================ }

constructor TMsgData.Create(AMSGID : string; ASize : longint; ARepNo : integer);
begin
inherited Create;
MSGID:=AMSGID;
Size:=ASize;
RepNo:=ARepNo;
Deleted:=false;
end;

end.
