unit Lsts;

interface

uses
 Classes, Logging, Windows;

type
 PHost = ^THost;
 THost = record
   Mask : string;
   AD   : boolean;
   end;

type
 THostList = class(TList)
   procedure Add(AMask : string; AAD : boolean);
 private
   function GetMask(i : integer) : string;
   function GetAD(i : integer) : boolean;
   function Match(AMask,AIP : string) : boolean;
 public
   property Mask[i : integer] : string read GetMask;
   property AD[i : integer] : boolean read GetAD;
   function Check(IP : string) : boolean;
   end;

 TDomainList = class(TStringList)
 private
   function MatchDomain(Base,Domain : string) : boolean;
 public
   function Match(ADomain : string) : boolean;
   end;

 TUserList = class(TStringList)
   procedure Add(AData : string); reintroduce;
   constructor Create;
 private
   Key : string;
   function GetData(i : integer) : string;
   function GetUsername(i : integer) : string;
   function GetPassword(i : integer) : string;
 public
   property Data[i : integer] : string read GetData;
   property Username[i : integer] : string read GetUsername;
   property Password[i : integer] : string read GetPassword;
   function MD5(AUsername,ATimestamp : string) : string;
   procedure AddUser(AUsername,APassword : string);
   function Check(AUsername,APassword : string) : boolean;
   end;

const
 HostList : THostList = nil;
 DomainList : TDomainList = nil;
 UserList : TUserList = nil;

implementation

uses
 Config, SysUtils, UpdateMD5, Reg;

{ ================================ THostList ================================= }

procedure THostList.Add(AMask : string; AAD : boolean);
var
 P : PHost;
begin
if empty(AMask) then exit;
new(P);
P.Mask:=AMask;
P.AD:=AAD;
inherited Add(P);
end;

function THostList.Match(AMask,AIP : string) : boolean;
var
 s1,s2 : string;
 i : integer;
begin
result:=false;
for i:=1 to 4 do
  begin
  s1:=GetIPPart(AMask,i);
  s2:=GetIPPart(AIP,i);
  if ((s1 <> '*') and (s1 <> s2)) or empty(s1) or empty(s2) then exit;
  end;
result:=true;
end;

function THostList.Check(IP : string) : boolean;
var
 a,d : boolean;
 i : integer;
begin
result:=false;
if empty(IP) then exit;
a:=false; d:=false;
for i:=0 to count-1 do if AD[i] then a:=true else d:=true;
if not a or (a and d) then result:=true;
if not a and not d then exit;
for i:=0 to count-1 do
  if not AD[i] then if Match(Mask[i],IP) then result:=false;
for i:=0 to count-1 do
  if AD[i] then if Match(Mask[i],IP) then result:=true;
end;

function THostList.GetMask(i : integer) : string;
begin
result:='';
if (i >= count) or (i < 0) then exit;
result:=PHost(items[i]).Mask;
end;

function THostList.GetAD(i : integer) : boolean;
begin
result:=false;
if (i >= count) or (i < 0) then exit;
result:=PHost(items[i]).AD;
end;

{ =============================== TDomainList ================================ }

function TDomainList.MatchDomain(Base,Domain : string) : boolean;
begin
result:=false;
if length(Base) > length(Domain) then exit;
if uppercase(Base) = uppercase(copy(Domain,length(Domain)-length(Base)+1,1000)) then result:=true;
end;

function TDomainList.Match(ADomain : string) : boolean;
var
 i : integer;
begin
result:=true;
for i:=0 to count-1 do
  if MatchDomain(strings[i],ADomain) then exit;
result:=false;
end;

{ ================================ TUserList ================================= }

procedure Crypt(var Buf : string; Key : string);
var
 i,j : integer;
begin
j:=1;
for i:=1 to length(Buf) do
  begin
  Buf[i]:=chr(byte(Buf[i]) xor byte(Key[j]));
  inc(j);
  if j > length(Key) then j:=1;
  end;
end;

const
 M : array[1..4] of integer = (555286440,559212551,1538430065,391692664);

constructor TUserList.Create;
begin
inherited Create;
Key:='';
// Variable C is the copyright string from the Reg.dcu unit (DCU subdirectory)
// You can use your own variable as the encryption key
// Sorry but I have to time to improve the encryption routine |-E
if uppercase(GetMD5Str(C[1],length(C))) = format('%X%X%X%X',[M[1],M[2],M[3],M[4]]) then
  Key:=GetMD5Str(C[1],length(C));
end;

procedure TUserList.Add(AData : string);
begin
Crypt(AData,Key);
inherited Add(AData);
end;

procedure TUserList.AddUser(AUsername,APassword : string);
begin
inherited Add(AUsername+'%'+APassword);
end;

function TUserList.GetData(i : integer) : string;
var
 s : string;
begin
if (i < 0) or (i >= count) then exit;
s:=strings[i];
Crypt(s,Key);
result:=s;
end;

function TUserList.GetUsername(i : integer) : string;
var
 j : integer;
begin
if (i < 0) or (i >= count) then exit;
j:=pos('%',strings[i]);
if j = 0 then
  result:=strings[i]
else
  result:=copy(strings[i],1,j-1);
end;

function TUserList.GetPassword(i : integer) : string;
var
 j : integer;
begin
if (i < 0) or (i >= count) then exit;
j:=pos('%',strings[i]);
if j = 0 then
  result:=''
else
  result:=copy(strings[i],j+1,1000);
end;

function TUserList.Check(AUsername,APassword : string) : boolean;
var
 i : integer;
begin
result:=true;
for i:=0 to count-1 do
  if uppercase(Username[i]) = uppercase(AUsername) then
    if Password[i] = APassword then exit;
result:=false;
end;

function TUserList.MD5(AUsername,ATimestamp : string) : string;
var
 i : integer;
 s : string;
begin
result:='';
for i:=0 to count-1 do
  if uppercase(Username[i]) = uppercase(AUsername) then
    begin
    s:=ATimestamp+Password[i];
    result:=GetMD5Str(s[1],length(s));
    end;
end;

end.
