unit Server;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Config, WinSock, WSAError, CBuf, Folder;

const
 MaxConn = 5;
 MaxOut = 250;

 ST_AUTH = 1;
 ST_TRANSACT = 2;
 ST_UPDATE = 3;

type
  TPopServer = class(tThread)
    procedure Execute; override;
    procedure Terminate;
  private
    sct   : integer;
    CB    : TCB;
    Error : integer;
    State : integer;
    nuser : string[20];
    Owner : TObject;
    Buf   : pchar;
    FLD   : TFolder;
    TS    : string;
    procedure ProcessCmd(cmd,param : string);
    procedure PostStr(msg : string);
    function TimeStamp : string;
    procedure Log(AMsg : string);
  public
    Run   : boolean;
    Index : integer;
    LAcc  : double;  // Last access time
    constructor create(AOwner : TObject; ASocket : integer; TrProc : TNotifyEvent);
  end;

  TServerList = class(TList)
    procedure Add(AServer : TPopServer);
    end;

  TTimer = class(TThread)
    constructor Create(ALst : TServerList);
    procedure Execute; override;
  private
    Lst : TServerList;
    end;

  TListener = class(TThread)
    constructor Create(AOwner : TObject);
    procedure Execute; override;
    procedure Terminate;
  private
    sct : integer;
    WD  : tWsaData;
    sin : tSockaddrIn;
    Lst : TServerList;
    Tm  : TTimer;
    Owner : TObject;
    Mtx : THandle;
    procedure ServerOnTerminate(Sender : TObject);
  public
    Run : boolean;
    procedure CLog(AMsg : string);
  end;

implementation

uses
 Logging, Main, Lsts;

{ ================================ TListener ================================= }

constructor TListener.Create(AOwner : TObject);
var
 prt : word;
 P   : pProtoEnt;
begin
inherited create(true);
FreeOnTerminate:=true;
Owner:=AOwner;
Run:=false;
Lst:=TServerList.Create;
Tm:=TTimer.Create(Lst);
WSAStartup($101,WD);
prt:=htons(Cfg.Port);
P:=GetProtoByName('TCP');
if P = nil then
  begin
  terminate;
  exit;
  end;
bzero(sin,sizeof(sin));
sin.sin_family:=AF_INET;
sin.sin_port:=prt;
sct:=socket(AF_INET,SOCK_STREAM,P.p_proto);
if sct < 0 then
  begin
  Log(FLM(1,WSAErrMsg(WSAGetLastError)));
  terminate;
  exit;
  end;
if bind(sct,tsockaddr(sin),sizeof(sin)) < 0 then
  begin
  Log(FLM(1,WSAErrMsg(WSAGetLastError)));
  terminate;
  exit;
  end;
if listen(sct,MaxConn) < 0 then
  begin
  Log(FLM(1,WSAErrMsg(WSAGetLastError)));
  terminate;
  exit;
  end;
Mtx:=CreateMutex(nil,false,nil);
Log(FLM(1,'POP3 server started'));
Run:=true;
Resume;
end;

procedure TListener.ServerOnTerminate(Sender : TObject);
begin
with Sender as TPopServer do
  begin
  Lst[Index]:=nil;
  end;
end;

type
 tba = array [1..4] of byte;

procedure TListener.Execute;
var
 len : integer;
 tsc : integer;
 sout: string;
begin
while not terminated do
  begin
  len:=sizeof(sin);
  tsc:=accept(sct,@tsockaddr(sin),@len);
  if tsc <= 0 then
    begin
    shutdown(tsc,2);
    closesocket(tsc);
    end
  else
    begin
    sout:=format('Connection request from %d.%d.%d.%d',
      [tba(sin.sin_addr)[1],tba(sin.sin_addr)[2],
      tba(sin.sin_addr)[3],tba(sin.sin_addr)[4]]);
    Log(FLM(1,sout));
    if HostList.Check(format('%u.%u.%u.%u',[tba(sin.sin_addr)[1],tba(sin.sin_addr)[2],tba(sin.sin_addr)[3],tba(sin.sin_addr)[4]])) then
      begin
      Lst.Add(TPopServer.Create(Self,tsc,ServerOnTerminate));
      end
    else
      begin
      Log(FLM(1,'Connection refused'));
      shutdown(tsc,2);
      closesocket(tsc);
      end;
    end;
  end;
end;

procedure TListener.Terminate;
var
 i : integer;
begin
for i:=0 to Lst.Count-1 do
  if assigned(Lst[i]) then TPopServer(Lst[i]).terminate;
Log(FLM(1,'POP3 server normal shutdown'));
CloseHandle(Mtx);
shutdown(sct,2);
closesocket(sct);
WSACleanup;
Run:=false;
inherited Terminate;
end;

procedure TListener.CLog(AMsg : string);
begin
WaitForSingleObject(Mtx,5000);
Logging.Log(AMsg);
ReleaseMutex(Mtx);
end;

{ =============================== TPopServer ================================= }

constructor TPopServer.Create;
begin
inherited create(true);
CB:=nil;
FLD:=nil;
Buf:=nil;
Owner:=AOwner;
Index:=-1;
sct:=ASocket;
Error:=0;
State:=st_AUTH;
nuser:='';
CB:=TCB.Create(2000);
GetMem(Buf,MaxOut+1);
FreeOnTerminate:=true;
OnTerminate:=TrProc;
LAcc:=now;
Resume;
end;

function TPopServer.TimeStamp : string;
var
 c : pchar;
 i : integer;
begin
GetMem(c,1000);
i:=999;
GetHostname(c,i);
result:=format('<%u.%u@%s>',[Application.Handle,round(now*10000),c]);
FreeMem(c);
end;

procedure TPopServer.Execute;
var
 s    : string;
 cmd  : string[20];
 par  : string[MaxOut];
 i    : integer;
begin
TS:=TimeStamp;
poststr(format('+OK POP3 server ready %s',[TS]));
while not terminated do
  begin
  while not CB.GetCB(s) do
    begin
    i:=recv(sct,Buf^,MaxOut,0);
    if i <= 0 then
      begin
      error:=WSAGetLastError;
      terminate;
      exit;
      end;
    LAcc:=now;
    Buf[i]:=#0;
    CB.AddCB(StrPas(Buf));
    end;
  cmd:=getword(s,1);
  par:=getend(s,2);
  ProcessCmd(cmd,par);
  end;
end;

procedure TPopServer.PostStr;
var
 s : string;
begin
Log(FLM(4,msg));
s:=msg+#13#10;
send(sct,s[1],length(s),0);
end;

procedure TPopServer.ProcessCmd;
var
 sout : string;
 i,i1 : longint;
 nmsg : longint;
begin
cmd:=uppercase(cmd);
sout:=format('cmd="%s", param="%s"',[cmd,param]);
Log(FLM(3,sout));
{--------------------------- QUIT ----------------------------}
if cmd = 'QUIT' then
  begin
  poststr('+OK Bye');
  if assigned(FLD) then
    begin
    FLD.Update;
    FLD.Free;
    FLD:=nil;
    end;
  terminate;
  end
{--------------------------- APOP ----------------------------}
else if cmd = 'APOP' then
  begin
  if State = ST_TRANSACT then
    begin
    poststr('-ERR Invalid command, valid commands are STAT, LIST, RETR, DELE,'+
            ' NOOP, RSET, TOP, UIDL, QUIT');
    end
  else
    begin
    nuser:=getword(param,1);
    if nuser = '' then
      poststr('-ERR Missing username')
    else
      begin
      if uppercase(getword(param,2)) = uppercase(UserList.MD5(nuser,TS)) then
        begin
        FLD:=TFolder.Create(nuser);
        if FLD.Error <> 0 then
          begin
          poststr('-ERR Can''t open mail folder. Try again later.');
          sout:=format('Can''t open folder file for user %s',[nuser]);
          Log(FLM(2,sout));
          end
        else
          begin
          sout:=format('+OK Mailbox opened (%d msgs)',[FLD.Count]);
          poststr(sout);
          State:=ST_TRANSACT;
          sout:=format('User %s logged in',[nuser]);
          Log(FLM(2,sout));
          end;
        end
      else
        begin
        sout:=format('Authentification error: user=%s, MD5=%s',[nuser,getword(param,2)]);
        Log(FLM(1,sout));
        poststr('-ERR Invalid Username/Password combination');
        end;
      end;
    end;
  end
{--------------------------- USER ----------------------------}
else if cmd = 'USER' then
  begin
  if Assigned(FLD) then FLD.Free;
  State:=ST_AUTH;
  nuser:=getword(param,1);
  if nuser = '' then
    poststr('-ERR Missing username')
  else
    poststr('+OK Send PASS command');
  end
{--------------------------- PASS ----------------------------}
else if cmd = 'PASS' then
  begin
  if State = ST_TRANSACT then
    begin
    poststr('-ERR Invalid command, valid commands are STAT, LIST, RETR, DELE,'+
            ' NOOP, RSET, TOP, UIDL, QUIT');
    end
  else
    begin
    if UserList.Check(nuser,param) then
      begin
      FLD:=TFolder.Create(nuser);
      if FLD.Error <> 0 then
        begin
        poststr('-ERR Can''t open mail folder. Try again later.');
        sout:=format('Can''t open folder file for user %s',[nuser]);
        Log(FLM(2,sout));
        end
      else
        begin
        sout:=format('+OK Mailbox opened (%d msgs)',[FLD.Count]);
        poststr(sout);
        State:=ST_TRANSACT;
        sout:=format('User %s logged in',[nuser]);
        Log(FLM(2,sout));
        end;
      end
    else
      begin
      sout:=format('Authentification error: user=%s, password=%s',[nuser,param]);
      Log(FLM(1,sout));
      poststr('-ERR Invalid Username/Password combination');
      end;
    end;
  end
{--------------------------- STAT ----------------------------}
else if cmd = 'STAT' then
  begin
  if State = ST_TRANSACT then
    begin
    sout:=format('+OK %d %d',[FLD.Count,FLD.Size]);
    PostStr(sout);
    end
  else
    begin
    poststr('-ERR Invalid command. Valid commands is USER, PASS, QUIT');
    end;
  end
{--------------------------- LIST ----------------------------}
else if cmd = 'LIST' then
  begin
  if State = ST_TRANSACT then
    begin
    if param = '' then
      begin
      poststr(format('+OK %u messages (%u octets)',[FLD.Count,FLD.Size]));
      for i1:=0 to FLD.Count-1 do
        begin
        if not FLD.MsgDeleted[i1] then
          poststr(format('%d %d',[i1+1,FLD.MsgSize[i1]]));
        end;
      poststr('.');
      end
    else
      begin
      i:=StrToIntDef(param,-1);
      if i < 0 then
        PostStr('-ERR message does not exist')
      else
        begin
        if not FLD.MsgDeleted[i] then
          begin
          sout:=format('+OK %d %d',[i,FLD.MsgSize[i]]);
          PostStr(sout);
          end
        else
          PostStr('-ERR message is deleted.');
        end;
      end;
    end
  else
    begin
    poststr('-ERR Invalid command. Valid commands is USER, PASS, QUIT');
    end;
  end
{--------------------------- RETR ----------------------------}
else if cmd = 'RETR' then
  begin
  if State = ST_TRANSACT then
    begin
    nmsg:=StrToIntDef(param,-1);
    if nmsg < 0 then
      PostStr('-ERR message does not exist')
    else
      begin
      if not FLD.MsgOpen(nmsg-1) then
        PostStr('-ERR message is deleted.')
      else
        begin
        PostStr('+OK message follows');
        sout:=format('POP3 recipient: %s',[nuser]);
        PostStr(sout);
        while FLD.MsgGetLine(sout) do
          begin
          if (length(sout) > 0) and (sout[1] = '.') then sout:='.'+sout;
          PostStr(sout);
          end;
        FLD.MsgClose;
        PostStr('.');
        end;
      end;
    end
  else
    begin
    poststr('-ERR Invalid command. Valid commands is USER, PASS, QUIT');
    end;
  end
{--------------------------- TOP ----------------------------}
else if cmd = 'TOP' then
  begin
  if State = ST_TRANSACT then
    begin
    nmsg:=StrToIntDef(getword(param,1),1);
    i:=StrToIntDef(getword(param,2),0);
    if nmsg < 0 then
      PostStr('-ERR message does not exist')
    else
      begin
      if not FLD.MsgOpen(nmsg) then
        PostStr('-ERR message is deleted')
      else
        begin
        PostStr('+OK message follows');
        sout:=format('POP3 recipient: %s',[nuser]);
        PostStr(sout);
        while FLD.MsgGetLine(sout) do
          begin
          PostStr(sout);
          if alltrim(sout) = '' then break;
          end;
        while FLD.MsgGetLine(sout) and (i > 0) do
          begin
          if (length(sout) > 0) and (sout[1] = '.') then sout:='.'+sout;
          PostStr(sout);
          dec(i);
          end;
        PostStr('.');
        end;
      end;
    end
  else
    begin
    poststr('-ERR Invalid command. Valid commands is USER, PASS, QUIT');
    end;
  end
{--------------------------- UIDL ----------------------------}
else if cmd = 'UIDL' then
  begin
  if State = ST_TRANSACT then
    begin
    if param = '' then
      begin
      poststr('+OK UID''s listing follows');
      for i1:=0 to FLD.Count-1 do
        begin
        if not FLD.MsgDeleted[i1] then
          begin
          sout:=format('%d %s',[i1+1,FLD.MsgMSGID[i1]]);
          poststr(sout);
          end;
        end;
      poststr('.');
      end
    else
      begin
      i:=StrToIntDef(param,-1)-1;
      if i < 0 then
        PostStr('-ERR message does not exist')
      else
        begin
        if not FLD.MsgDeleted[i] then
          begin
          sout:=format('+OK %d %s',[i+1,FLD.MsgMSGID[i]]);
          PostStr(sout);
          end
        else
          PostStr('-ERR message is deleted');
        end;
      end;
    end
  else
    begin
    poststr('-ERR Invalid command. Valid commands is USER, PASS, QUIT');
    end;
  end
{--------------------------- DELE ----------------------------}
else if cmd = 'DELE' then
  begin
  if State = ST_TRANSACT then
    begin
    nmsg:=StrToIntDef(param,-1);
    if nmsg < 0 then
      PostStr('-ERR message does not exist')
    else
      begin
      if FLD.MsgDelete(nmsg-1) then
        poststr('+OK message deleted')
      else
        PostStr('-ERR message is already deleted.');
      end;
    end
  else
    begin
    poststr('-ERR Invalid command. Valid commands is USER, PASS, QUIT');
    end;
  end
{--------------------------- NOOP ----------------------------}
else if cmd = 'NOOP' then
  begin
  if State = ST_TRANSACT then
    begin
    PostStr('+OK');
    end
  else
    begin
    poststr('-ERR Invalid command. Valid commands is USER, PASS, QUIT');
    end;
  end
{--------------------------- RSET ----------------------------}
else if cmd = 'RSET' then
  begin
  if State = ST_TRANSACT then
    begin
    FLD.Reset;
    PostStr(format('+OK maildrop has %u messages (%u octets)',[FLD.Count,FLD.Size]));
    end
  else
    begin
    poststr('-ERR Invalid command. Valid commands is USER, PASS, QUIT');
    end;
  end
else
  begin
  poststr('-ERR Unknown command');
  end;
end;

procedure tPopServer.Terminate;
begin
shutdown(sct,2);
closesocket(sct);
if assigned(Buf) then
  begin
  FreeMem(Buf);
  Buf:=nil;
  end;
if assigned(CB) then
  begin
  CB.Free;
  CB:=nil;
  end;
if assigned(Fld) then
  begin
  FLD.Free;
  FLD:=nil;
  end;
inherited Terminate;
end;

procedure TPopServer.Log(AMsg : string);
begin
TListener(Owner).CLog(AMsg);
end;

{ =============================== TServerList =================================}

procedure TServerList.Add(AServer : TPopServer);
var
 i : integer;
begin
for i:=0 to count-1 do
  if not assigned(items[i]) then
    begin
    items[i]:=AServer;
    AServer.Index:=i;
    exit;
    end;
inherited Add(AServer);
AServer.Index:=count-1;
end;

{ ================================== TTimer ===================================}

constructor TTimer.Create(ALst : TServerList);
begin
inherited Create(true);
Lst:=ALst;
Resume;
end;

procedure TTimer.Execute;
var
 i : integer;
 d : double;
begin
while not terminated do
  begin
  sleep(1000);
  d:=now;
  for i:=0 to Lst.Count-1 do
    begin
    if assigned(Lst[i]) then
      begin
      if (d-TPopServer(Lst[i]).LAcc) > (Cfg.TOut*OneSecond) then
        TPopServer(Lst[i]).Terminate;
      end;
    end;
  end;
end;

end.
